@@ -9,6 +9,7 @@ import abc
 import asyncio
 from dataclasses import dataclass
 from genericpath import exists
+from operator import truediv
 from typing import Dict, Optional, Sequence, Tuple, TypeVar
 from enum import Enum
 from xmlrpc.client import Boolean
@@ -18,6 +19,10 @@ import datetime
 
 _LOGGER = logging.getLogger(__name__)
 
+class Model(Enum):
+    SSA_3220 = 'SSA-3200'
+    SSA_3220D = 'SSA-3220D'
+
 class Connection:
     def __init__(self, host: str, port: int):
         self._host = host
@@ -118,7 +123,7 @@ class Input:
        
     async def updated(self):
         _LOGGER.info(f'Output {self._number} Updated: {self}')
-        await self._switch._updated("updated", self)
+        await self._switch._updated("input-updated", self)
 
     async def parse(self, key: str, value: str):
         if key == 'trim':
@@ -190,7 +195,7 @@ class Output:
    
     async def updated(self):
         _LOGGER.info(f'Output {self._number} Updated: {self}')
-        await self._switch._updated("updated", self)
+        await self._switch._updated("output-updated", self)
 
     async def parse(self, key: str, value: str):
         _LOGGER.debug(f"Output.parse({key} => {value}")
@@ -218,6 +223,8 @@ class Output:
             await self._switch.send_command(f'aoutput-delayboth-get{self._number}')
     
     async def set_volume(self, vol: int):
+        if vol < -38 or vol > 0:
+            raise ValueError(f'Invalid volume level: {vol}dB')
         await self._switch.send_command(f'aoutput-vol-set{self._number}:{vol}dB')
         pass
     
@@ -256,8 +263,8 @@ class Switch:
         self._links = {} # output -> input
         self._callback = None
         self._connection = Connection(self._host, self._port)
-        self._fwrev = None
-        self._fpgarev = None
+        self._attributes = {}
+        self._ready = False
         self._model = model
         if self._model == 'SSA-3220' or self._model == 'SSA-3220D':
             self._ninputs = 32
@@ -274,12 +281,12 @@ class Switch:
         return self._port
     
     @property
-    def fwrev(self):
-        return self._fwrev
+    def attributes(self):
+        return self._attributes
     
     @property
-    def fpgarev(self):
-        return self._fpgarev
+    def model(self):
+        return self._model
     
     async def connect(self):
         await self.refresh()
@@ -325,12 +332,33 @@ class Switch:
         async for reply in self._connection.send('fwrev'):
             m = re.search('fwrevPrimary: (.*)', reply)
             if m:
-                self._fwrev = m.group(1)
+                self._attributes['fwrev'] = m.group(1)
 
         async for reply in self._connection.send('fpga-rev'):
             m = re.search('fpga-rev(.*)', reply)
             if m:
-                self._fpgarev = m.group(1)
+                self._attributes['fpgarev'] = m.group(1)
+        
+        async for reply in self._connection.send('status'):
+            m = re.search('statusAPI1.0; (.*)', reply)
+            if m:
+                for part in m.group(1).split(';'):
+                    part = part.trim()
+                    if part.startswith('pn'):
+                        self._attributes['pn'] = part
+                    elif part.startswith('sn'):
+                        self._attributes['sn'] = part
+                    elif part.startswith('rev'):
+                        self._attributes['rev'] = part
+                    elif part == 'ready=yes':
+                        self._ready = True
+                    elif part == 'ready=no':
+                        self._ready = False
+                    elif part == 'Standalone-Audio-Switch-With-Delay':
+                        self._model = Model.SSA_3220D
+                    elif part == 'Standalone-Audio-Switch':
+                        self._model = Model.SSA_3220
+
 
         for c in range(1, self._noutputs):
             await self.refresh_link(c)
@@ -363,17 +391,21 @@ class Switch:
     def outputs(self):
         return [output for output in self._outputs if output is not None]
 
-    async def link(self, input: int, output: int):
+    async def link(self, output: int, input: int):
         await self.send_command(f'switch-set{output}.{input}')
 
-    async def unlink(self, input: int, output: int):
-        link = await self.get_link(output)
-        if link is not None and link == input:
+    async def unlink(self, output: int, input: int = None):
+        if input is None:
+            #unlink no matter what
             await self.send_command(f'switch-set{output}.disconnect')
+        else:
+            link = await self.get_link(output)
+            if link is not None and link == input:
+                await self.send_command(f'switch-set{output}.disconnect')
 
     async def link_changed(self, output: int, input: int):
         _LOGGER.info(f'switch {output} connected to {input}')
-        await self._updated('changed', (output, input))
+        await self._updated('link-changed', (output, input))
     
     @property
     def links(self):
